@@ -22,7 +22,13 @@
 #include <linux/ext_fs.h>
 #endif
 
+#ifdef HAVE_LINUX_EXT2_FS_H
 #include <linux/ext2_fs.h>
+#endif
+
+#ifdef HAVE_EXT2FS_EXT2_FS_H
+#include <ext2fs/ext2_fs.h>
+#endif
 
 #ifdef HAVE_LINUX_XIA_FS_H
 #include <linux/xia_fs.h>
@@ -629,7 +635,7 @@ int do_mount(char *devname,char *_mountpoint,
     struct mntent ms;
     struct floppy_drive_struct drivstat;
     char options[80+MAX_OPT]; 
-    char super[2048];
+    unsigned char super[2048];
     char *mountpoint;
    
     strncpy(curdev,devname, sizeof(curdev));
