@@ -21,6 +21,7 @@ typedef enum
 	FE_STRETCH,
 	FE_SWAPSIDES,
 	FE_ZEROBASED,
+	FE_FIRSTSECTORNUMBER,
 
 	FE_GAP,
 
@@ -36,7 +37,7 @@ typedef enum
 	FE_DENSITY,
 } field_t;
 
-static int SIZE, SECT, VSECT, HEAD, CYL, TPI, STRETCH, SWAPSIDES, ZEROBASED;
+static int SIZE, SECT, VSECT, HEAD, CYL, TPI, STRETCH, SWAPSIDES, ZEROBASED, FIRSTSECTORNUMBER=1;
 static int GAP, FM;
 static int PERP, SSIZE, _2M, DTR, SPEC1, FMT_GAP, DENSITY;
 static int ssize;
@@ -51,6 +52,7 @@ static int ssize;
 #define F_STRETCH FE_STRETCH,&STRETCH
 #define F_SWAPSIDES FE_SWAPSIDES,&SWAPSIDES
 #define F_ZEROBASED FE_ZEROBASED,&ZEROBASED
+#define F_FIRSTSECTORNUMBER FE_FIRSTSECTORNUMBER,&FIRSTSECTORNUMBER
 
 #define F_GAP FE_GAP,&GAP
 
@@ -82,6 +84,7 @@ static keyword_t ids[]= {
 	{ "swapsides", F_SWAPSIDES, 1},
 	{ "zerobased", F_ZEROBASED, 1},
 	{ "zero-based", F_ZEROBASED, 1},
+	{ "first-sector-number", F_FIRSTSECTORNUMBER, 0},
 
 	{ "gap", F_GAP, 0},
 
@@ -235,11 +238,25 @@ static void compute_params(drivedesc_t *drvprm,
 	}
 	set_field(F_2M,0);
 
+	if(mask & (1 << FE_ZEROBASED)) {
+		if((mask & (1 << FE_FIRSTSECTORNUMBER)) &&
+		   FIRSTSECTORNUMBER != 0) {
+			fprintf(stderr,
+				"Zerobased incompatible with first-sector-number=%d\n",
+				FIRSTSECTORNUMBER);
+			exit(1);
+		} else
+			FIRSTSECTORNUMBER = 0;
+	} else if(! (mask & (1 << FE_FIRSTSECTORNUMBER))) {
+		FIRSTSECTORNUMBER = 1;
+	}
+
 	medprm->size = SIZE;
 	medprm->sect = VSECT / 512;
 	medprm->head = HEAD;
 	medprm->track = CYL;
-	medprm->stretch = STRETCH | (SWAPSIDES << 1) | (ZEROBASED << 2);
+	medprm->stretch = STRETCH | (SWAPSIDES << 1) | 
+	  ((1^FIRSTSECTORNUMBER) << 2);
 	medprm->gap = GAP;
 	medprm->rate = (FM<<7) | (PERP<<6) | (ssize<<3) | (_2M<<2) | DTR;
 	medprm->spec1 = SPEC1;
@@ -301,8 +318,6 @@ int parse_mediaprm(int argc, char **argv,
 		   drivedesc_t *drvprm,
 		   struct floppy_struct *medprm)		  
 {
-	int r;
-	
-	return ((r=parse_direct(argc, argv, drvprm, medprm) &&
-		 (r=parse_indirect(argc, argv, drvprm, medprm))));
+	return parse_direct(argc, argv, drvprm, medprm) &&
+		parse_indirect(argc, argv, drvprm, medprm);
 }
