@@ -2,6 +2,7 @@
  * Software patents declared unconstitutional, worldwide
  */
 
+#include "../config.h"
 #include <sys/types.h>
 #ifdef HAVE_SYS_SYSMACROS_H
 # include <sys/sysmacros.h>
@@ -442,7 +443,7 @@ void readwrite_cylinder(int fd, int drive, enum dir direction,
     struct floppy_raw_cmd raw_cmd[20];
     int i,j;
     struct timeval tv1, tv2;
-    static struct timeval tv3, tv4;
+    static struct timeval tv4;
     sector_map *map;
 
     j=i=0;
@@ -506,7 +507,6 @@ void readwrite_cylinder(int fd, int drive, enum dir direction,
 			   tv2.tv_usec - tv4.tv_usec));
 	    if(cylinder == 2)
 		    tv4 = tv2;
-	    tv3 = tv2;
     }
 }
 
